/*
 * Projet Système de particules
 *
 * ancre.c
 *
 *  Fonctions et type ancre.
 *
 *
 */

#include <stdlib.h>
#include "list.h"
#include "geometrie.h"
#include "particule.h"
#include "ancre.h"

/** Précision de la recherche */
#define DELTA 4


/* Fonction de création d'une ancre
 * Arguments:
 * 	particule: La particule qui sera ancrée
 * 	x: Abscisse de l'ancre
 * 	y: Ordonnée de l'ancre
 * Retourne: Un pointeur sur une ancre
 */
Ancre* creer_ancre(Particule* particule, double x, double y) {
	/* Allocation de mémoire */
	Ancre* ancre = (Ancre*)malloc(sizeof(Ancre));

	/* Affectation des valeurs */
	ancre -> particule = particule;
	ancre -> position = creer_point(x,y);

	return ancre;
}

/** Procédure de résolution de contrainte de type "ancre"
 * Argument: L'ancre dont on veut résoudre la contrainte
 */
void resoudre_contrainte_ancre(Ancre* ancre) {
	Particule* particule = ancre -> particule;
	Point* position = ancre -> position;

	/* Pour une ancre: On replace la particule à sa position originelle */
	particule -> position -> x = position -> x;
	particule -> position -> y = position -> y;
}
	 

/** Procédure détruisant l'ancre
 * Argument: ancre: Ancre à détruire
 */
void detruire_ancre(Ancre** ancre) {
	/* On doit juste détruire le point, et non la particule */
	free((*ancre) -> position); (*ancre) -> position = NULL;

	/* Destruction de l'ancre */
	free(*ancre); (*ancre) = NULL;
}

/** Procédure utilisée pour l'appel à detruire_ancre dans l'itérateur
 * Argument: ancre: Ancre à détruire
 */
void detruire_ancre_it(Ancre* ancre) {
	detruire_ancre(&ancre);
}

/** Fonction d'égalité de deux ancres par leur position
 * Arguments:
 * 	a1: Première ancre
 * 	a2: Seconde ancre
 * Retourne: position a1 == position a2
 */
int egalite_ancre_pos(Ancre* a1, Ancre* a2) {
	return (abs(a1 -> position -> x - a2 -> position -> x) <= DELTA) && 
		(abs(a1 -> position -> y -  a2 -> position -> y) <= DELTA);
}
