/**
 * Projet Système de particules
 *
 * ancre.h
 *
 *  Fonctions et type ancre.
 *
 *
 */

#ifndef _H_ANCRE
#define _H_ANCRE

/* Définition du type particule */
typedef struct {
	Particule* particule; /* La particule à ancrer */
	Point* position; /* Sa position */
	} Ancre;

/* Fonction de création d'une ancre
 * Arguments:
 * 	particule: Pointeur sur la particule qui sera ancrée
 * 	x: Abscisse de l'ancre
 * 	y: Ordonnée de l'ancre
 * Retourne: Un pointeur sur une ancre
 */
Ancre* creer_ancre(Particule* particule, double x, double y);

/** Procédure de résolution de contrainte de type "ancre"
 * Argument: L'ancre dont on veut résoudre la contrainte
 */
void resoudre_contrainte_ancre(Ancre* ancre);

/** Procédure détruisant l'ancre
 * Argument: ancre: Ancre à détruire
 */
void detruire_ancre(Ancre** ancre);

/** Procédure utilisée pour l'appel à detruire_ancre dans l'itérateur
 * Argument: ancre: Ancre à détruire
 */
void detruire_ancre_it(Ancre* ancre);

/** Fonction d'égalité de deux ancres par leur position
 * Arguments:
 * 	a1: Première ancre
 * 	a2: Seconde ancre
 * Retourne: position a1 == position a2
 */
int egalite_ancre_pos(Ancre* a1, Ancre* a2);

#endif
