/**
 * Projet Système de particules
 *
 * chaine.c
 *
 * Implémentation du type chaine.
 *
 */

#include <stdlib.h>
#include <math.h>
#include "list.h"
#include "geometrie.h"
#include "particule.h"
#include "chaine.h"

/** Fonction d'ajout d'une chaine
 * Arguments:
 *	p1: Première particule
 *	p2: Seconde particule
 *	l: Longueur de la chaine
 * Retourne: La chaine qui vient d'être créée
 */
Chaine* creer_chaine(Particule* p1, Particule* p2, double l) {
	/* Allocation de la mémoire */
	Chaine* chaine = (Chaine*)malloc(sizeof(Chaine));

	/* Affectation des membres */
	chaine -> e1 = p1;
	chaine -> e2 = p2;

	chaine -> longueur = l;

	return chaine;
}

/** Procédure de résolution de contrainte de type "chaine"
 * Argument: La chaine dont on veut résoudre la contrainte
 */
void resoudre_contrainte_chaine(Chaine* chaine) {
	Point* pt1 = chaine -> e1 -> position;
	Point* pt2 = chaine -> e2 -> position;

	/* On récupère l'inverse des masses des particules */
	double invm1 = chaine -> e1 -> masse;
	double invm2 = chaine -> e2 -> masse; 
	
	/* Variables servant pour les calculs */
	double delta_x, delta_y;
	double delta;
	double diff;

	/* On ne modifie la distance que si les deux particules sont éloignées
	 * de plus de la longueur de la chaine.
	 */
	if(distance(pt1,pt2) > chaine -> longueur) {	
		/* On calcule la distance entre les deux points */
		delta_x = pt2 -> x - pt1 -> x;
		delta_y = pt2 -> y - pt1 -> y;

		/* delta est la norme du vecteur pt2 - pt1 */
		delta = sqrt(pow(delta_x,2)+pow(delta_y,2));

		/* La correction à appliquer aux points pour donner longueur fixée */
		diff = (delta - chaine -> longueur)/(delta*(invm1 + invm2));

		/* On corrige le premier point */
		pt1 -> x += invm1*delta_x*diff;
		pt1 -> y += invm1*delta_y*diff;

		/* On corrige le second point */
		pt2 -> x -= invm2*delta_x*diff;
		pt2 -> y -= invm2*delta_y*diff;
	}
}

/** Procédure détruisant la chaine
 * Argument: chaine: Chaine à détruire
 */
void detruire_chaine(Chaine** chaine) {
	free(*chaine); (*chaine) = NULL;
}

/** Procédure utilisée pour convertir l'appel à detruire_chainee dans
 * l'itérateur
 * Argument: chaine: La chaine à détruire
 */
void detruire_chaine_it(Chaine* chaine) {
	detruire_chaine(&chaine);
}
