/*
 * Projet Système de particules
 *
 * geometrie.c
 *
 *  Fonctions et types géométriques utiles.
 *
 *
 */
#include <stdlib.h>
#include <math.h>
#include "geometrie.h"

/** Fonction de création d'un point
 * Arguments:
 * 	double x :  L'abscisse du point
 * 	double y :  L'ordonnée du point
 * Retourne: Un pointeur vers un Point
 */
Point* creer_point (double x, double y){
	/* On alloue la mémoire */
	Point* point = (Point*)malloc(sizeof(Point));
	
	/* Affectation des valeurs */
	point -> x = x;
	point -> y = y;

	return point;
}

/** Fonction calculant la distance entre deux points
 * Arguments:
 * 	p1: Premier point
 * 	p2: Deuxieme point
 * Retourne: La distance entre p1 et p2
 */
double distance (Point* p1, Point* p2) {
	double dx = p2 -> x - p1 -> x;
	double dy = p2 -> y - p1 -> y;

	return sqrt(pow(dx,2)+pow(dy,2));
}

/** Fonction de création d'un vecteur
 * Arguments:
 * 	nx: Norme en x
 * 	ny: Norme en y
 * Retourne: Un pointeur sur un vecteur
 */
Vecteur* creer_vecteur (double nx, double ny) {
	/* Allocation de mémoire */
	Vecteur* vect = (Vecteur*)malloc(sizeof(Vecteur));

	/* Affectation des valeurs */
	vect -> norme_x = nx;
	vect -> norme_y = ny;

	return vect;
}

/** Fonction retournant la norme d'un vecteur
 * Argument: Un vecteur
 * Retourne: La norme du vecteur
 */
double norme(Vecteur* vect) {
	double x = vect -> norme_x;
	double y = vect -> norme_y; 

	return sqrt(x*x+y*y);
}

/** Procédure détruisant un vecteur
 * Argument: vect: Le vecteur à détruire
*/
void detruire_vecteur(Vecteur** vecteur) {
	free(*vecteur); vecteur = NULL;
}

/** Procédure utilisée lors de l'appel à detruire_vecteur dans l'itérateur
 * Argument: vect: Vecteur à détruire
*/
void detruire_vecteur_it(Vecteur* vect) {
	detruire_vecteur(&vect);
}

