/*
 * Projet Système de particules
 *
 * interaction.h 
 *
 * Fonctions de manipulation de l'IHM
 *
 */

#ifndef __INTERACTION_H__
#define __INTERACTION_H__

#include "geometrie.h"

/*
 * L'utilisateur a cliqué (bouton enfoncé) dans la fenêtre au point (x,y)
 */
extern void on_viewport_mouse_press(double x, double y);

/*
 * Le bouton de la souris a été relaché.
 */
extern void on_viewport_mouse_release();

/*
 * Autorise/Désactive les événements cliquer-glisser.
 * Appelé lorsque un object a été sélectionné
 */
extern void viewport_activate_drag(int activate);

/*
 * L'utilisateur a fait un cliquer-glisser dans la fenêtre.
 *
 * Le cliqué est en (orig_x, orig_y) et la destination
 * (position courante de la souris) est (dest_x, dest_y). 
 *
 */
extern void on_viewport_mouse_drag(double orig_x, double orig_y,
                                   double dest_x, double dest_y);

/*
 * Efface la fenêtre d'affichage (blanc)
 */
extern void viewport_clear();

/*
 * Dessine un point aux coordonnées (x,y)
 * dans la fenêtre.
 */
extern void viewport_draw_point(double x, double y, int size);

/*
 * Dessine un segment entre les points (x1,y1) et (x2,y2)
 * width est l'épaisseur de la ligne.
 */
extern void viewport_draw_line(double x1, double y1, double x2, double y2, int width);

/*
 * Dessine un polygone fermé constitué des points données.
 */
extern void viewport_draw_polygon(Point *points, int nb_points, int plein, int line_width);

/*
 * Change la couleur des dessins en cours
 * red:   proportion de rouge (entre 0.0 et 1.0)
 * blue:  proportion de bleu (entre 0.0 et 1.0)
 * green: proportion de vert (entre 0.0 et 1.0)
 */
extern void viewport_change_color(double red, double green, double blue);

#endif
