/* Interface du module Liste Chaînée Polymorphe  qui définit 
 le type abstrait list dont les éléments sont de type (void *)                */

#ifndef _LIST
  #define _LIST
  typedef struct node *T_LIST;
  /* Les valeurs de retour */
  #define USAGE_PARAM       -1
  #define NO_ERROR           0
  #define ALLOC_ERROR        1

/* fonction pour récupérer une liste vide 
    l = nil()                                                                 */
extern T_LIST nil();

/* fonction pour ajouter un élément en tête d'une liste l
    ret = cons( (void *) a, &l)
   avec ret = ALLOC_ERROR si un problème d'allocation se pose,
              NO_ERROR sinon                                                  */
extern int cons(void *val, T_LIST *l);

/* fonction pour tester la liste                                              */
extern int is_empty(T_LIST l); 

/* fonction pour récupérer l'élément en tête de liste l
   val = (int *) head(l, &ret) récupère le pointeur vers l'élément en
   tête (ici pour l'EXEMPLE un entier)
   avec ret = USAGE_PARAM si la liste est vide et dans ce cas val est NULL
              NO_ERROR sinon                                                  */
extern void* head(T_LIST l, int *ret);

/* fonction pour récupérer le reste de la liste l
   q = tail(l, &ret) 
   avec ret = USAGE_PARAM si la liste est vide et dans ce cas q est NULL
              NO_ERROR sinon                                                  */
extern T_LIST tail(T_LIST l, int *ret);

/* fonction pour récupérer le label dans la liste l égal au sens
   de la fonction equal à val ;
   equal(a, b) renvoie 0 si ce qui est pointé par a est différent
   de ce qui est pointé par b.
   si la recherche échoue, le résultat est NULL.
   exemple : pi = (int*) member(qi, l, eq_int)                                */
extern void *member(void *val, T_LIST l, int (*equal)());

/* fonction pour insérer au bon endroit au sens de la fonction cmp
   dans une liste triée l, la valeur val ;
   si doublon est 0, ce sera une insertion sans doublon,
   avec doublon sinon ;
   le résultat est ALLOC_ERROR si un problème d'allocation apparaît,
                   NO_ERROR sinon.
   exemple ret = insert_in_sorted_list(&l, (void*) pi, 1, cmp_int)            */
extern int insert_in_sorted_list(T_LIST *l, void *val, 
                          int doublon, int (*cmp)());

/* procédure d'application de la procédure p à tous les labels de la liste l
   exemple do_list(aff_int, l)                                                */
extern void do_list(void (*p)(), T_LIST l);

#endif
