/*
 * Projet Système de particules
 *
 * ressort.h
 *
 *  Fonctions et type ressort.
 *
 *
 */

#ifndef _H_RESSORT
#define _H_RESSORT

/* Définition du type Ressort */
typedef struct {
	Particule* e1; /* Première extrémité du ressort */
	Particule* e2; /* Seconde extrémité */
	double longueur; /* Longueur au repos du ressort */
	double coefficient; /* Coefficient de raideur du ressort */
	} Ressort;


/** Fonction créant un ressort de longueur l, de coefficient k
 *et d'extrémité p1 et p2
 * Arguments:
 * 	p1: Première extrémité
 * 	p2: Seconde extrémité
 * 	l: Longueur de la tige
 * 	k: Coefficient de raideur du ressort
 * Retourne: Un pointeur sur le ressort créé
 */
Ressort* creer_ressort(Particule* p1, Particule* p2, double l, double k);

/** Procédure de calcul ds forces d'appliquant sur chacune des extrémités du
 * ressort
 */
void calculer_force_ressort(Ressort* ressort);

/** Procédure détruisant le ressort
 * Argument: ressort: Ressort à détruire
 */
void detruire_ressort(Ressort** ressort);

/** Procédure utilisée pour convertir l'appel à detruire_ressort dans
 * l'itérateur
 * Argument: ressort: Le ressort à détruire
 */
void detruire_ressort_it(Ressort* ressort);

#endif
