/*
 * Projet Système de particules
 *
 * tige.h
 *
 *  Fonctions et type tige.
 *
 *
 */

#ifndef _H_TIGE
#define _H_TIGE

/* Définition du type Tige */
typedef struct {
	Particule* e1; /* La première extrémité */
	Particule* e2; /* La seconde extrémité */
	double longueur; /* La longueur de la tige */
	} Tige;


/** Fonction créant une tige de longueur l et d'extrémité p1 et p2
 * Arguments:
 * 	p1: Première extrémité de la tige
 * 	p2: Seconde extrémité de la tige
 * 	l: Longueur de la tige
 * Retourne: Un pointeur sur la tige créée
 */
Tige* creer_tige(Particule* p1, Particule* p2, double l);

/** Procédure de résolution de contrainte de type "tige"
 * Argument: La tige dont on veut résoudre la contrainte
 */
void resoudre_contrainte_tige(Tige* tige);

/** Procédure détruisant la tige
 * Argument: tige: Tige à détruire
 */
void detruire_tige(Tige** tige);

/** Procédure utilisée pour convertir l'appel à detruire_tigee dans
 * l'itérateur
 * Argument: tige: La tige à détruire
 */
void detruire_tige_it(Tige* tige);

/** Fonction d'égalité de deux tiges par la position d'une de leur extrémité
 * Arguments:
 * 	t1: Première tige
 * 	t2: Seconde tige
 * Retourne: (extremite 1 de t1 == extremite 1 de t2) || (extremite 2 t1 ==
 * extremite 2 de t2)
 */
int egalite_tige_ext(Tige* t1, Tige* t2);

#endif
